@@ -103,10 +103,11 @@ class SocialAuthBackend(ModelBackend):
         details = self.get_user_details(response)
         uid = self.get_user_id(details, response)
         is_new = False
+        user = kwargs.get('user')
+
         try:
             social_user = self.get_social_auth_user(uid)
         except UserSocialAuth.DoesNotExist:
-            user = kwargs.get('user')
             if user is None:  # new user
                 if not CREATE_USERS:
                     return None
@@ -134,12 +135,11 @@ class SocialAuthBackend(ModelBackend):
             # at this moment, merging account is not an option because that
             # would imply update user references on other apps, that's too
             # much intrusive
-            if 'user' in kwargs and kwargs['user'] != social_user.user:
+            if user and user != social_user.user:
                 raise ValueError('Account already in use.', social_user)
             user = social_user.user
 
-        # Update user account data.
-        self.update_user_details(user, response, details, is_new)
+        # Flag user "new" status
         setattr(user, 'is_new', is_new)
 
         # Update extra_data storage, unless disabled by setting
@@ -150,6 +150,9 @@ class SocialAuthBackend(ModelBackend):
                 social_user.save()
 
         user.social_user = social_user
+
+        # Update user account data.
+        self.update_user_details(user, response, details, is_new)
         return user
 
     def username(self, details):
